"use strict";
/*********************************
 File:       xlogger.ts
 Function:   Xlogger in typescript
 Copyright:  Bertco LLC
 Date:       2020-02-14
 Author:     mkahn
 **********************************/
Object.defineProperty(exports, "__esModule", { value: true });
exports.log = exports.debug = exports.error = exports.warn = exports.info = exports.verbose = exports.silly = exports.sentry = exports.reactotron = exports.logDebug = exports.logError = exports.logWarn = exports.logInfo = exports.logVerbose = exports.logSilly = exports.out = exports.setUseCorrespondingConsoleMethod = exports.setUseSentry = exports.setUseReactotron = exports.setLogLevel = exports.configure = exports.setReactronInstance = void 0;
var Reactolog = require("./reactolog");
var SentryLog = require("./sentry");
var types_1 = require("./types");
var helpers_1 = require("./helpers");
var DEFAULT_CONFIG = {
    logLevel: types_1.LogLevel.debug,
    useCorrespondingConsoleMethod: true,
    reactotronInstance: undefined,
    useReactotron: false,
    useSentry: false,
    printLogLevel: true,
    printLogTime: false,
};
var currentConfig = DEFAULT_CONFIG;
var setReactronInstance = function (instance) {
    currentConfig.reactotronInstance = instance;
    Reactolog.setReactotronInstance(instance);
};
exports.setReactronInstance = setReactronInstance;
var configure = function (settings) {
    currentConfig = {
        logLevel: (settings === null || settings === void 0 ? void 0 : settings.logLevel) || DEFAULT_CONFIG.logLevel,
        useCorrespondingConsoleMethod: (settings === null || settings === void 0 ? void 0 : settings.useCorrespondingConsoleMethod) ||
            DEFAULT_CONFIG.useCorrespondingConsoleMethod,
        reactotronInstance: (settings === null || settings === void 0 ? void 0 : settings.reactotronInstance) || DEFAULT_CONFIG.reactotronInstance,
        useReactotron: (settings === null || settings === void 0 ? void 0 : settings.useReactotron) || DEFAULT_CONFIG.useReactotron,
        useSentry: (settings === null || settings === void 0 ? void 0 : settings.useSentry) || DEFAULT_CONFIG.useSentry,
        printLogLevel: (settings === null || settings === void 0 ? void 0 : settings.printLogLevel) || DEFAULT_CONFIG.printLogLevel,
        printLogTime: (settings === null || settings === void 0 ? void 0 : settings.printLogTime) || DEFAULT_CONFIG.printLogTime
    };
    if (settings === null || settings === void 0 ? void 0 : settings.reactotronInstance) {
        Reactolog.setReactotronInstance(settings.reactotronInstance);
    }
};
exports.configure = configure;
var appendPrefixes = function (message, logLevel) {
    var t = typeof message; // instanceof doesn't work on literals
    if (t === 'string' || t === 'number') {
        var llPrefix = currentConfig.printLogLevel ? "[" + helpers_1.descriptionForLevel(logLevel) + "]" : '';
        var now = new Date();
        var ltPrefix = currentConfig.printLogTime ?
            "[" + now.getHours() + ":" + now.getMinutes() + ":" + now.getSeconds() + "." + now.getMilliseconds() + "]" : '';
        return "" + ltPrefix + llPrefix + "  " + message;
    }
    // no prefixes on objects
    return message;
};
var applyDefaultParams = function (params) {
    // fixme: who the heall is calling without params? How?
    if (!params) {
        return {
            bypassReactotron: true,
            bypassSentry: true,
        };
    }
    return {
        bypassReactotron: !!params.bypassReactotron,
        bypassSentry: !!params.bypassSentry,
    };
};
var logIfLevelLegit = function (message, bypassParams, level) {
    var _a = applyDefaultParams(bypassParams), bypassReactotron = _a.bypassReactotron, bypassSentry = _a.bypassSentry;
    if (level <= currentConfig.logLevel) {
        if (level === types_1.LogLevel.error && currentConfig.useCorrespondingConsoleMethod) {
            // eslint-disable-next-line no-console
            console.error(appendPrefixes(message, level));
        }
        else if (level === types_1.LogLevel.warn && currentConfig.useCorrespondingConsoleMethod) {
            // eslint-disable-next-line no-console
            console.warn(appendPrefixes(message, level));
        }
        else {
            // eslint-disable-next-line no-console
            console.log(appendPrefixes(message, level));
        }
        if (currentConfig.reactotronInstance && !bypassReactotron) {
            Reactolog.log(message);
        }
        if (currentConfig.useSentry && !bypassSentry) {
            SentryLog.log(message, level);
        }
    }
};
/**
 * Sets the logLevel, if it is in the list of legit ones above.
 * @param level
 */
var setLogLevel = function (level) {
    currentConfig.logLevel = level;
};
exports.setLogLevel = setLogLevel;
/**
 * Enables/disables Reactotron logging
 * @param shouldUse
 */
var setUseReactotron = function (shouldUse) {
    currentConfig.useReactotron = shouldUse;
};
exports.setUseReactotron = setUseReactotron;
/**
 * Enables/disables Sentry logging
 * @param shouldUse
 */
var setUseSentry = function (shouldUse) {
    currentConfig.useSentry = shouldUse;
};
exports.setUseSentry = setUseSentry;
/**
 * Maps logWarn to console.warn, and logError to console.error
 * @param shouldUse
 */
var setUseCorrespondingConsoleMethod = function (shouldUse) {
    currentConfig.useCorrespondingConsoleMethod = shouldUse;
};
exports.setUseCorrespondingConsoleMethod = setUseCorrespondingConsoleMethod;
/**
 * Always log out, bypass log level unless silent.
 * @param message
 * @param bypassParams
 */
var out = function (message, bypassParams) {
    if (bypassParams === void 0) { bypassParams = { bypassSentry: true, bypassReactotron: false }; }
    if (currentConfig.logLevel !== types_1.LogLevel.silent) {
        // eslint-disable-next-line no-console
        console.log(message);
        if (currentConfig.useReactotron && !bypassParams.bypassReactotron) {
            Reactolog.log(message);
        }
        if (currentConfig.useSentry && !bypassParams.bypassSentry) {
            SentryLog.log(message, types_1.LogLevel.info);
        }
    }
};
exports.out = out;
var logSilly = function (message, bypassParams) {
    if (bypassParams === void 0) { bypassParams = { bypassReactotron: false, bypassSentry: true }; }
    return logIfLevelLegit(message, bypassParams, types_1.LogLevel.silly);
};
exports.logSilly = logSilly;
var logVerbose = function (message, bypassParams) {
    if (bypassParams === void 0) { bypassParams = { bypassReactotron: false, bypassSentry: true }; }
    return logIfLevelLegit(message, bypassParams, types_1.LogLevel.verbose);
};
exports.logVerbose = logVerbose;
var logInfo = function (message, bypassParams) {
    if (bypassParams === void 0) { bypassParams = { bypassReactotron: false, bypassSentry: true }; }
    return logIfLevelLegit(message, bypassParams, types_1.LogLevel.info);
};
exports.logInfo = logInfo;
// for Warn level, default is not to bypass sentry
var logWarn = function (message, bypassParams) {
    if (bypassParams === void 0) { bypassParams = { bypassReactotron: false, bypassSentry: false }; }
    return logIfLevelLegit(message, bypassParams, types_1.LogLevel.warn);
};
exports.logWarn = logWarn;
var logError = function (message, bypassParams) {
    if (bypassParams === void 0) { bypassParams = { bypassReactotron: false, bypassSentry: false }; }
    return logIfLevelLegit(message, bypassParams, types_1.LogLevel.error);
};
exports.logError = logError;
var logDebug = function (message, bypassParams) {
    if (bypassParams === void 0) { bypassParams = { bypassReactotron: false, bypassSentry: true }; }
    return logIfLevelLegit(message, bypassParams, types_1.LogLevel.debug);
};
exports.logDebug = logDebug;
// direct access, only if turned on.
exports.reactotron = {
    log: function (message) {
        if (currentConfig.useReactotron) {
            Reactolog.log(message);
        }
    },
    logImportant: function (message) {
        if (currentConfig.useReactotron) {
            Reactolog.logImportant(message);
        }
    },
    display: function (object) {
        if (currentConfig.useReactotron) {
            Reactolog.display(object);
        }
    },
};
exports.sentry = {
    log: SentryLog.log,
    logFatal: SentryLog.logFatal,
    logCritical: SentryLog.logCritical
};
// synonyms
exports.silly = exports.logSilly;
exports.verbose = exports.logVerbose;
exports.info = exports.logInfo;
exports.warn = exports.logWarn;
exports.error = exports.logError;
exports.debug = exports.logDebug;
// backwards compatible, kind of, with the old confusing `log` method.
exports.log = exports.logDebug;
//# sourceMappingURL=xlogger.js.map